@@ -5,17 +5,18 @@ use prost_types::{FileDescriptorProto, MethodDescriptorProto, ServiceDescriptorP
 use crate::descriptor::debug_fmt_iter;
 
 use super::{
-    make_full_name, parse_name, parse_namespace, ty, DescriptorError, FileDescriptor,
-    MessageDescriptor,
+    make_full_name, parse_name, parse_namespace, ty, DescriptorError, DescriptorPool,
+    MessageDescriptor, MethodIndex, ServiceIndex,
 };
 
 /// A protobuf service definition.
 #[derive(Clone, PartialEq, Eq)]
 pub struct ServiceDescriptor {
-    file_descriptor: FileDescriptor,
-    index: u32,
+    descriptor_pool: DescriptorPool,
+    index: ServiceIndex,
 }
 
+#[derive(Clone)]
 pub(super) struct ServiceDescriptorInner {
     full_name: Box<str>,
     methods: Box<[MethodDescriptorInner]>,
@@ -25,9 +26,10 @@ pub(super) struct ServiceDescriptorInner {
 #[derive(Clone, PartialEq, Eq)]
 pub struct MethodDescriptor {
     service: ServiceDescriptor,
-    index: u32,
+    index: MethodIndex,
 }
 
+#[derive(Clone)]
 struct MethodDescriptorInner {
     full_name: Box<str>,
     request_ty: ty::TypeId,
@@ -37,27 +39,27 @@ struct MethodDescriptorInner {
 }
 
 impl ServiceDescriptor {
-    /// Create a new [`ServiceDescriptor`] referencing the service at `index` within the given [`FileDescriptor`].
+    /// Create a new [`ServiceDescriptor`] referencing the service at `index` within the given [`DescriptorPool`].
     ///
     /// # Panics
     ///
     /// Panics if `index` is out-of-bounds.
-    pub fn new(file_descriptor: FileDescriptor, index: usize) -> Self {
-        debug_assert!(index < file_descriptor.services().len());
+    pub fn new(descriptor_pool: DescriptorPool, index: usize) -> Self {
+        debug_assert!(index < descriptor_pool.services().len());
         ServiceDescriptor {
-            file_descriptor,
+            descriptor_pool,
             index: index.try_into().expect("index too large"),
         }
     }
 
-    /// Returns the index of this [`ServiceDescriptor`] within the parent [`FileDescriptor`].
+    /// Returns the index of this [`ServiceDescriptor`] within the parent [`DescriptorPool`].
     pub fn index(&self) -> usize {
         self.index as usize
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this service is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
-        &self.file_descriptor
+    /// Gets a reference to the [`DescriptorPool`] this service is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
+        &self.descriptor_pool
     }
 
     /// Gets the short name of the service, e.g. `MyService`.
@@ -81,10 +83,8 @@ impl ServiceDescriptor {
     pub fn service_descriptor_proto(&self) -> &ServiceDescriptorProto {
         let name = self.name();
         let package = self.package_name();
-        self.parent_file()
-            .file_descriptor_set()
-            .file
-            .iter()
+        self.parent_pool()
+            .file_descriptor_protos()
             .filter(|file| file.package() == package)
             .flat_map(|file| file.service.iter())
             .find(|service| service.name() == name)
@@ -97,7 +97,7 @@ impl ServiceDescriptor {
     }
 
     fn inner(&self) -> &ServiceDescriptorInner {
-        &self.parent_file().inner.services[self.index as usize]
+        &self.parent_pool().inner.services[self.index as usize]
     }
 }
 
@@ -160,9 +160,9 @@ impl MethodDescriptor {
         &self.service
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this method is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
-        self.service.parent_file()
+    /// Gets a reference to the [`DescriptorPool`] this method is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
+        self.service.parent_pool()
     }
 
     /// Gets the short name of the method, e.g. `method`.
@@ -182,12 +182,12 @@ impl MethodDescriptor {
 
     /// Gets the [`MessageDescriptor`] for the input type of this method.
     pub fn input(&self) -> MessageDescriptor {
-        MessageDescriptor::new(self.parent_file().clone(), self.inner().request_ty)
+        MessageDescriptor::new(self.parent_pool().clone(), self.inner().request_ty)
     }
 
     /// Gets the [`MessageDescriptor`] for the output type of this method.
     pub fn output(&self) -> MessageDescriptor {
-        MessageDescriptor::new(self.parent_file().clone(), self.inner().response_ty)
+        MessageDescriptor::new(self.parent_pool().clone(), self.inner().response_ty)
     }
 
     /// Returns `true` if the client streams multiple messages.
