@@ -7,7 +7,7 @@ use std::{
 use prost::bytes::Bytes;
 use prost_types::{
     field_descriptor_proto::{self, Label},
-    DescriptorProto, EnumDescriptorProto, FieldDescriptorProto, FileDescriptorSet,
+    DescriptorProto, EnumDescriptorProto, FieldDescriptorProto, FileDescriptorProto,
 };
 
 use crate::{
@@ -18,13 +18,13 @@ use crate::{
             FieldDescriptorInner, MessageDescriptorInner, OneofDescriptorInner, ParentKind, TypeId,
             TypeMap,
         },
-        MAP_ENTRY_KEY_NUMBER, MAP_ENTRY_VALUE_NUMBER,
+        EnumValueIndex, FileIndex, MAP_ENTRY_KEY_NUMBER, MAP_ENTRY_VALUE_NUMBER,
     },
     DescriptorError,
 };
 
 impl TypeMap {
-    pub fn add_files(&mut self, raw: &FileDescriptorSet) -> Result<(), DescriptorError> {
+    pub fn add_files(&mut self, raw: &[FileDescriptorProto]) -> Result<(), DescriptorError> {
         let mut messages = Vec::new();
         let mut enums = Vec::new();
         let mut extensions = Vec::new();
@@ -270,13 +270,13 @@ impl TypeMap {
             .collect();
         values.sort_by_key(|v| v.number);
 
-        let value_names: HashMap<Box<str>, u32> = values
+        let value_names: HashMap<Box<str>, EnumValueIndex> = values
             .iter()
             .enumerate()
             .map(|(index, value)| {
                 (
                     value.name.clone(),
-                    u32::try_from(index).expect("index too large"),
+                    EnumValueIndex::try_from(index).expect("index too large"),
                 )
             })
             .collect();
@@ -360,12 +360,12 @@ impl TypeMap {
 
     fn iter_files<'a>(
         &mut self,
-        raw: &'a FileDescriptorSet,
+        raw: &'a [FileDescriptorProto],
         messages: &mut Vec<MessageProto<'a>>,
         enums: &mut Vec<EnumProto<'a>>,
         extensions: &mut Vec<ExtensionProto<'a>>,
     ) -> Result<(), DescriptorError> {
-        for (file_index, file) in raw.file.iter().enumerate() {
+        for (file_index, file) in raw.iter().enumerate() {
             let file_index = file_index.try_into().expect("index too large");
             let syntax = match file.syntax.as_deref() {
                 None | Some("proto2") => Syntax::Proto2,
@@ -433,7 +433,7 @@ impl TypeMap {
     #[allow(clippy::too_many_arguments)]
     fn iter_message<'a>(
         &mut self,
-        file_index: u32,
+        file_index: FileIndex,
         namespace: &str,
         messages: &mut Vec<MessageProto<'a>>,
         enums: &mut Vec<EnumProto<'a>>,
@@ -504,7 +504,7 @@ enum Syntax {
 
 #[derive(Clone)]
 struct MessageProto<'a> {
-    file: u32,
+    file: FileIndex,
     full_name: Box<str>,
     message_proto: &'a DescriptorProto,
     parent: Option<Box<str>>,
@@ -513,7 +513,7 @@ struct MessageProto<'a> {
 
 #[derive(Clone)]
 struct EnumProto<'a> {
-    file: u32,
+    file: FileIndex,
     full_name: Box<str>,
     enum_proto: &'a EnumDescriptorProto,
     parent: Option<Box<str>>,
@@ -522,7 +522,7 @@ struct EnumProto<'a> {
 
 #[derive(Clone)]
 struct ExtensionProto<'a> {
-    file: u32,
+    file: FileIndex,
     namespace: Box<str>,
     field_proto: &'a FieldDescriptorProto,
     parent: Option<Rc<str>>,
