@@ -19,13 +19,16 @@ use prost_types::{
 };
 
 use crate::descriptor::{
-    debug_fmt_iter, make_full_name, parse_name, parse_namespace, DescriptorError, FileDescriptor,
+    debug_fmt_iter, make_full_name, parse_name, parse_namespace, DescriptorError, DescriptorPool,
     MAP_ENTRY_KEY_NUMBER, MAP_ENTRY_VALUE_NUMBER,
 };
 
+use super::{EnumIndex, EnumValueIndex, ExtensionIndex, FileIndex, MessageIndex, OneofIndex};
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub(super) struct TypeId(field_descriptor_proto::Type, u32);
 
+#[derive(Clone, Default)]
 pub(super) struct TypeMap {
     named_types: HashMap<Box<str>, TypeId>,
     messages: Vec<MessageDescriptorInner>,
@@ -36,10 +39,11 @@ pub(super) struct TypeMap {
 /// A protobuf message definition.
 #[derive(Clone, PartialEq, Eq)]
 pub struct MessageDescriptor {
-    file_set: FileDescriptor,
-    index: u32,
+    file_set: DescriptorPool,
+    index: MessageIndex,
 }
 
+#[derive(Clone)]
 struct MessageDescriptorInner {
     full_name: Box<str>,
     parent: ParentKind,
@@ -48,16 +52,17 @@ struct MessageDescriptorInner {
     field_names: HashMap<Box<str>, u32>,
     field_json_names: HashMap<Box<str>, u32>,
     oneof_decls: Box<[OneofDescriptorInner]>,
-    extensions: Vec<u32>,
+    extensions: Vec<ExtensionIndex>,
 }
 
 /// A oneof field in a protobuf message.
 #[derive(Clone, PartialEq, Eq)]
 pub struct OneofDescriptor {
     message: MessageDescriptor,
-    index: u32,
+    index: OneofIndex,
 }
 
+#[derive(Clone)]
 struct OneofDescriptorInner {
     name: Box<str>,
     full_name: Box<str>,
@@ -71,6 +76,7 @@ pub struct FieldDescriptor {
     field: u32,
 }
 
+#[derive(Clone)]
 struct FieldDescriptorInner {
     name: Box<str>,
     full_name: Box<str>,
@@ -87,10 +93,11 @@ struct FieldDescriptorInner {
 /// A protobuf extension field definition.
 #[derive(Clone, PartialEq, Eq)]
 pub struct ExtensionDescriptor {
-    file_set: FileDescriptor,
-    index: u32,
+    file_set: DescriptorPool,
+    index: ExtensionIndex,
 }
 
+#[derive(Clone)]
 pub struct ExtensionDescriptorInner {
     field: FieldDescriptorInner,
     number: u32,
@@ -102,25 +109,27 @@ pub struct ExtensionDescriptorInner {
 /// A protobuf enum type.
 #[derive(Clone, PartialEq, Eq)]
 pub struct EnumDescriptor {
-    file_set: FileDescriptor,
-    index: u32,
+    file_set: DescriptorPool,
+    index: EnumIndex,
 }
 
+#[derive(Clone)]
 struct EnumDescriptorInner {
     full_name: Box<str>,
     parent: ParentKind,
-    value_names: HashMap<Box<str>, u32>,
+    value_names: HashMap<Box<str>, EnumValueIndex>,
     values: Vec<EnumValueDescriptorInner>,
-    default_value: u32,
+    default_value: EnumValueIndex,
 }
 
 /// A value in a protobuf enum type.
 #[derive(Clone, PartialEq, Eq)]
 pub struct EnumValueDescriptor {
     parent: EnumDescriptor,
-    index: u32,
+    index: EnumValueIndex,
 }
 
+#[derive(Clone)]
 struct EnumValueDescriptorInner {
     name: Box<str>,
     number: i32,
@@ -179,12 +188,12 @@ pub enum Cardinality {
 
 #[derive(Copy, Clone, Debug)]
 enum ParentKind {
-    File { index: u32 },
-    Message { index: u32 },
+    File { index: FileIndex },
+    Message { index: MessageIndex },
 }
 
 impl MessageDescriptor {
-    pub(in crate::descriptor) fn new(file_set: FileDescriptor, ty: TypeId) -> Self {
+    pub(in crate::descriptor) fn new(file_set: DescriptorPool, ty: TypeId) -> Self {
         debug_assert_eq!(ty.0, field_descriptor_proto::Type::Message);
         MessageDescriptor {
             file_set,
@@ -193,7 +202,7 @@ impl MessageDescriptor {
     }
 
     pub(in crate::descriptor) fn iter(
-        file_set: &FileDescriptor,
+        file_set: &DescriptorPool,
     ) -> impl ExactSizeIterator<Item = Self> + '_ {
         file_set
             .inner
@@ -203,7 +212,7 @@ impl MessageDescriptor {
     }
 
     pub(in crate::descriptor) fn try_get_by_name(
-        file_set: &FileDescriptor,
+        file_set: &DescriptorPool,
         name: &str,
     ) -> Option<Self> {
         let ty = file_set.inner.type_map.get_by_name(name)?;
@@ -213,8 +222,8 @@ impl MessageDescriptor {
         Some(MessageDescriptor::new(file_set.clone(), ty))
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this message is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
+    /// Gets a reference to the [`DescriptorPool`] this message is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
         &self.file_set
     }
 
@@ -252,7 +261,7 @@ impl MessageDescriptor {
 
     /// Gets a reference to the raw [`DescriptorProto`] wrapped by this [`MessageDescriptor`].
     pub fn descriptor_proto(&self) -> &DescriptorProto {
-        find_message_descriptor_proto(self.parent_file(), self.index)
+        find_message_descriptor_proto(self.parent_pool(), self.index)
     }
 
     /// Gets an iterator yielding a [`FieldDescriptor`] for each field defined in this message.
@@ -410,9 +419,9 @@ impl fmt::Debug for MessageDescriptor {
 }
 
 impl FieldDescriptor {
-    /// Gets a reference to the [`FileDescriptor`] this field is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
-        self.message.parent_file()
+    /// Gets a reference to the [`DescriptorPool`] this field is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
+        self.message.parent_pool()
     }
 
     /// Gets a reference to the [`MessageDescriptor`] this field is defined in.
@@ -550,7 +559,7 @@ impl fmt::Debug for FieldDescriptor {
 
 impl ExtensionDescriptor {
     pub(in crate::descriptor) fn iter(
-        file_set: &FileDescriptor,
+        file_set: &DescriptorPool,
     ) -> impl ExactSizeIterator<Item = Self> + '_ {
         file_set
             .inner
@@ -562,8 +571,8 @@ impl ExtensionDescriptor {
             })
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this extension field is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
+    /// Gets a reference to the [`DescriptorPool`] this extension field is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
         &self.file_set
     }
 
@@ -792,7 +801,7 @@ impl fmt::Debug for Kind {
 }
 
 impl EnumDescriptor {
-    pub(in crate::descriptor) fn new(file_set: FileDescriptor, ty: TypeId) -> Self {
+    pub(in crate::descriptor) fn new(file_set: DescriptorPool, ty: TypeId) -> Self {
         debug_assert_eq!(ty.0, field_descriptor_proto::Type::Enum);
         EnumDescriptor {
             file_set,
@@ -801,7 +810,7 @@ impl EnumDescriptor {
     }
 
     pub(in crate::descriptor) fn iter(
-        file_set: &FileDescriptor,
+        file_set: &DescriptorPool,
     ) -> impl ExactSizeIterator<Item = Self> + '_ {
         file_set
             .inner
@@ -811,7 +820,7 @@ impl EnumDescriptor {
     }
 
     pub(in crate::descriptor) fn try_get_by_name(
-        file_set: &FileDescriptor,
+        file_set: &DescriptorPool,
         name: &str,
     ) -> Option<Self> {
         let ty = file_set.inner.type_map.get_by_name(name)?;
@@ -821,8 +830,8 @@ impl EnumDescriptor {
         Some(EnumDescriptor::new(file_set.clone(), ty))
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this enum type is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
+    /// Gets a reference to the [`DescriptorPool`] this enum type is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
         &self.file_set
     }
 
@@ -959,9 +968,9 @@ impl EnumValueDescriptor {
         }
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this enum value is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
-        self.parent.parent_file()
+    /// Gets a reference to the [`DescriptorPool`] this enum value is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
+        self.parent.parent_pool()
     }
 
     /// Gets a reference to the [`EnumDescriptor`] this enum value is defined in.
@@ -1017,9 +1026,9 @@ impl OneofDescriptor {
         }
     }
 
-    /// Gets a reference to the [`FileDescriptor`] this oneof is defined in.
-    pub fn parent_file(&self) -> &FileDescriptor {
-        self.message.parent_file()
+    /// Gets a reference to the [`DescriptorPool`] this oneof is defined in.
+    pub fn parent_pool(&self) -> &DescriptorPool {
+        self.message.parent_pool()
     }
 
     /// Gets a reference to the [`MessageDescriptor`] this oneof is defined in.
@@ -1069,15 +1078,6 @@ impl fmt::Debug for OneofDescriptor {
 }
 
 impl TypeMap {
-    pub fn new() -> Self {
-        TypeMap {
-            named_types: HashMap::new(),
-            messages: Vec::new(),
-            enums: Vec::new(),
-            extensions: Vec::new(),
-        }
-    }
-
     pub fn shrink_to_fit(&mut self) {
         self.named_types.shrink_to_fit();
         self.messages.shrink_to_fit();
@@ -1131,7 +1131,7 @@ impl TypeMap {
         }
     }
 
-    fn get_message(&self, index: u32) -> &MessageDescriptorInner {
+    fn get_message(&self, index: MessageIndex) -> &MessageDescriptorInner {
         &self.messages[index as usize]
     }
 
@@ -1140,11 +1140,11 @@ impl TypeMap {
         &mut self.messages[ty.1 as usize]
     }
 
-    fn get_enum(&self, index: u32) -> &EnumDescriptorInner {
+    fn get_enum(&self, index: EnumIndex) -> &EnumDescriptorInner {
         &self.enums[index as usize]
     }
 
-    fn get_extension(&self, index: u32) -> &ExtensionDescriptorInner {
+    fn get_extension(&self, index: ExtensionIndex) -> &ExtensionDescriptorInner {
         &self.extensions[index as usize]
     }
 
@@ -1216,7 +1216,7 @@ impl TypeId {
         }
     }
 
-    fn to_kind(self, file_set: &FileDescriptor) -> Kind {
+    fn to_kind(self, file_set: &DescriptorPool) -> Kind {
         match self.0 {
             field_descriptor_proto::Type::Double => Kind::Double,
             field_descriptor_proto::Type::Float => Kind::Float,
@@ -1255,7 +1255,7 @@ impl ParentKind {
 
     fn resolve_parent_file_proto<'a>(
         &self,
-        file_set: &'a FileDescriptor,
+        file_set: &'a DescriptorPool,
     ) -> &'a FileDescriptorProto {
         let mut curr = *self;
         loop {
@@ -1269,11 +1269,14 @@ impl ParentKind {
     }
 }
 
-fn get_file_descriptor_proto(file_set: &FileDescriptor, index: u32) -> &FileDescriptorProto {
-    &file_set.file_descriptor_set().file[index as usize]
+fn get_file_descriptor_proto(file_set: &DescriptorPool, index: FileIndex) -> &FileDescriptorProto {
+    file_set
+        .file_descriptor_protos()
+        .nth(index as usize)
+        .unwrap()
 }
 
-fn find_message_descriptor_proto(file_set: &FileDescriptor, index: u32) -> &DescriptorProto {
+fn find_message_descriptor_proto(file_set: &DescriptorPool, index: FileIndex) -> &DescriptorProto {
     let message = file_set.inner.type_map.get_message(index);
     match message.parent {
         ParentKind::File { index: file_index } => get_file_descriptor_proto(file_set, file_index)
