@@ -20,12 +20,12 @@ use crate::{
         },
         DynamicMessage,
     },
-    FileDescriptor,
+    DescriptorPool,
 };
 
 use super::{deserialize_message, kind::MessageVisitorInner, MessageSeed};
 
-pub struct GoogleProtobufAnyVisitor<'a>(pub &'a FileDescriptor, pub &'a DeserializeOptions);
+pub struct GoogleProtobufAnyVisitor<'a>(pub &'a DescriptorPool, pub &'a DeserializeOptions);
 pub struct GoogleProtobufNullVisitor;
 pub struct GoogleProtobufTimestampVisitor;
 pub struct GoogleProtobufDurationVisitor;
