@@ -1,3 +1,4 @@
+mod case;
 mod de;
 mod ser;
 
@@ -15,7 +16,7 @@ pub struct SerializeOptions {
     stringify_64_bit_integers: bool,
     use_enum_numbers: bool,
     use_proto_field_name: bool,
-    emit_unpopulated_fields: bool,
+    skip_default_fields: bool,
 }
 
 /// Options to control deserialization of messages.
@@ -119,7 +120,7 @@ impl SerializeOptions {
             stringify_64_bit_integers: true,
             use_enum_numbers: false,
             use_proto_field_name: false,
-            emit_unpopulated_fields: false,
+            skip_default_fields: true,
         }
     }
 
@@ -154,14 +155,14 @@ impl SerializeOptions {
         self
     }
 
-    /// Whether to emit unpopulated fields.
+    /// Whether to skip fields which have their default value.
     ///
-    /// If `false`, any fields for which [`has_field`][DynamicMessage::has_field] returns `false` will
-    /// not be serialized. If `true`, they will be serialized with their default value.
+    /// If `true`, any fields for which [`has_field`][DynamicMessage::has_field] returns `false` will
+    /// not be serialized. If `false`, they will be serialized with their default value.
     ///
-    /// The default value is `false`.
-    pub const fn emit_unpopulated_fields(mut self, yes: bool) -> Self {
-        self.emit_unpopulated_fields = yes;
+    /// The default value is `true`.
+    pub const fn skip_default_fields(mut self, yes: bool) -> Self {
+        self.skip_default_fields = yes;
         self
     }
 }
@@ -172,6 +173,12 @@ impl Default for SerializeOptions {
     }
 }
 
+const MAX_DURATION_SECONDS: u64 = 315_576_000_000;
+const MAX_DURATION_NANOS: u32 = 999_999_999;
+
+const MIN_TIMESTAMP_SECONDS: i64 = -62135596800;
+const MAX_TIMESTAMP_SECONDS: i64 = 253402300799;
+
 fn is_well_known_type(full_name: &str) -> bool {
     matches!(
         full_name,
