@@ -8,7 +8,7 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![warn(missing_debug_implementations, missing_docs)]
 #![deny(unsafe_code)]
-#![doc(html_root_url = "https://docs.rs/prost-reflect/0.6.0/")]
+#![doc(html_root_url = "https://docs.rs/prost-reflect/0.8.0/")]
 
 #[cfg(feature = "serde1")]
 extern crate serde1 as serde;
@@ -17,10 +17,12 @@ mod descriptor;
 mod dynamic;
 mod reflect;
 
+pub use {prost, prost::bytes, prost_types};
+
 pub use self::descriptor::{
-    Cardinality, DescriptorError, EnumDescriptor, EnumValueDescriptor, ExtensionDescriptor,
-    FieldDescriptor, FileDescriptor, Kind, MessageDescriptor, MethodDescriptor, OneofDescriptor,
-    ServiceDescriptor,
+    Cardinality, DescriptorError, DescriptorPool, EnumDescriptor, EnumValueDescriptor,
+    ExtensionDescriptor, FieldDescriptor, Kind, MessageDescriptor, MethodDescriptor,
+    OneofDescriptor, ServiceDescriptor,
 };
 pub use self::dynamic::{DynamicMessage, MapKey, Value};
 pub use self::reflect::ReflectMessage;
